/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#pragma  hdrstop

#include <stdlib.h>
#include <string.h>
#include "account.h"


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
Account::Account( char *psFile)
  {
  pStocks    = NULL;
  pFunds     = NULL;
  pTransacts = NULL;

  AddBogusInfo( psFile);
  }

Account::~Account( )
  {
  if (pFunds)
    pFunds->DeleteAll( );
  if (pStocks)
    pStocks->DeleteAll( );
  if (pTransacts)
    pTransacts->DeleteAll( );
  }

MutualFund *Account::GetFund( FILE *pf)
  {
  char  s[301];

  if (fgets( s, sizeof( s), pf) == NULL)
    return NULL;

  if (strcmp( s, "Stocks\n") == 0)
    return NULL;

  MutualFund *pMF = new MutualFund;

  if (pMF != NULL)
    {
    char *sp = strtok( s, ",");      strcpy( pMF->sName, sp);
    sp = strtok( NULL, ",");         strcpy( pMF->sSymbol, sp);
    sp = strtok( NULL, ",");         strcpy( pMF->sPurchDate, sp);
    sp = strtok( NULL, ",");         pMF->dPurchQty   = atof( sp);
    sp = strtok( NULL, ",");         pMF->dPurchPrice = atof( sp);
    sp = strtok( NULL, ",");         pMF->dCurrQty    = atof( sp);
    sp = strtok( NULL, ",");         pMF->dCurrPrice  = atof( sp);
    }

  return pMF;
  }

Stock *Account::GetStock( FILE *pf)
  {
  char  s[301];

  if (fgets( s, sizeof( s), pf) == NULL)
    return NULL;

  if (strcmp( s, "Transactions\n") == 0)
    return NULL;

  Stock *pS = new Stock;

  if (pS != NULL)
    {
    char *sp = strtok( s, ",");      strcpy( pS->sName, sp);
    sp = strtok( NULL, ",");         strcpy( pS->sSymbol, sp);
    sp = strtok( NULL, ",");         strcpy( pS->sPurchDate, sp);
    sp = strtok( NULL, ",");         pS->dPurchQty   = atof( sp);
    sp = strtok( NULL, ",");         pS->dPurchPrice = atof( sp);
    sp = strtok( NULL, ",");         pS->dCurrQty    = atof( sp);
    sp = strtok( NULL, ",");         pS->dCurrPrice  = atof( sp);
    }

  return pS;
  }

Transaction *Account::GetTransaction( FILE *pf)
  {
  char  s[301];

  if (fgets( s, sizeof( s), pf) == NULL)
    return NULL;

  Transaction *pT = new Transaction;

  if (pT != NULL)
    {
    char *sp = strtok( s, ",");      strcpy( pT->sDate, sp);
    sp = strtok( NULL, ",");         strcpy( pT->sDesc, sp);
    sp = strtok( NULL, ",");         pT->dQty     = atof( sp);
    sp = strtok( NULL, ",");         pT->dPrice   = atof( sp);
    sp = strtok( NULL, ",");         pT->dAmount  = atof( sp);
    sp = strtok( NULL, ",");         pT->dBalance = atof( sp);
    }

  return pT;
  }

void Account::AddBogusInfo( char *psFile)
  {
  FILE *pf = fopen( psFile, "rt");

  char  s[101];
  fgets( s, sizeof( s), pf);     //-- throw away first line

  MutualFund  *pFund;
  while (!feof( pf))
    {
    pFund = GetFund( pf);
    if (pFund == NULL)
      break;

    pFund->AddAtEnd( (NodeDoubleLink *&)pFunds);
    }

  Stock *pStock;
  while (!feof( pf))
    {
    pStock= GetStock( pf);
    if (pStock == NULL)
      break;

    pStock->AddAtEnd( (NodeDoubleLink *&)pStocks);
    }

  Transaction *pT;
  while (!feof( pf))
    {
    pT = GetTransaction( pf);
    if (pT == NULL)
      break;

    pT->AddAtEnd( (NodeDoubleLink *&)pTransacts);
    }

  fclose( pf);
  }

double Account::TotalCostBasis( )
  {
  double d = 0;
  for (MutualFund *pMF=pFunds; pMF!=NULL; pMF=pMF->Next( ))
    d += pMF->dPurchPrice * pMF->dPurchQty;
  for (Stock *pS=pStocks; pS!=NULL; pS=pS->Next( ))
    d += pS->dPurchPrice * pS->dPurchQty;

  return d;
  }

double Account::TotalStocks( )
  {
  double d = 0;
  for (Stock *pS=pStocks; pS!=NULL; pS=pS->Next( ))
    d += pS->dCurrPrice * pS->dCurrQty;

  return d;
  }

double Account::TotalMutualFunds( )
  {
  double d = 0;
  for (MutualFund *pMF=pFunds; pMF!=NULL; pMF=pMF->Next( ))
    d += pMF->dCurrPrice * pMF->dCurrQty;

  return d;
  }

double Account::TotalCurrValue( )
  {
  return TotalStocks( ) + TotalMutualFunds( ) + TotalCash( ) +
         TotalCDs( ) +    TotalCorpBonds( ) +   TotalGovBonds( );
  }
