/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#include "orcc.h"
#include <windows.h>
#pragma  hdrstop

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "oracct.h"
#include "ordemo.h"
#include "orfont.h"
#include "orprntdc.h"
#include "ortext.h"


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
static void DollarValue( char *psDest, double d)
  {
  sprintf( psDest, "$%0.2f", d);

  //---- search for first digit (in case of leading spaces or '$', etc.)
  char *pc = psDest; 
  while ((*pc) && (!isdigit( *pc)))
    pc++;
  if (*pc == 0)
    return;

  char *psStart = pc;

  //---- search for last digit
  while ((*pc) && (isdigit( *pc))) 
    pc++;

  pc -= 3;
  while (pc > psStart)
    {
    memmove( (pc + 1), pc, (strlen( pc) + 1));
    *pc = ',';
    pc -= 3;
    }
  }

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_AHeader::ORLayer_AHeader( ORReport_Account *pRepAcct_, long lAttr_, 
                                  Account *pAcct_)
                :ORLayer( pRepAcct_, lAttr_)
  {
  pRepAcct = pRepAcct_;
  pAcct    = pAcct_;
  }

long ORLayer_AHeader::InstantiateChildren( )
  {
  long lErr = 0;

  ORLayer *pL = new ORLayer( this);
  new ORText( pL, 1, "Account:\nName:", IDFntA_Arial10,
                                        (ORPOS_LEFT | ORPOS_TOP | ORJUST_RIGHT));

  char  s[51];
  sprintf( s, "%s\n%s", pAcct->AccountNo( ), pAcct->Name( ));
  new ORText( pL, 0, s, IDFntA_Arial10, ORPOS_RIGHT, 1);

  sprintf( s, "%s\n%s", pRepAcct->DateBegin( ), pRepAcct->DateEnd( ));
  new ORText( pL, 2, s, IDFntA_Arial10, (ORPOS_RIGHT | ORPOS_TOP | ORJUST_LEFT));
  new ORText( pL, 0, "Period Beg:\nEnd:", IDFntA_Arial10,
                                          (ORPOS_LEFT | ORJUST_RIGHT), 2);

  new ORText( pL, 0, "Dewey, Skrooim & Howe\n"
                     "Asset Management Statement",
                     IDFntA_Arial18, ORPOS_CENTER);

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*-----------------------------------------------------------------------------
  This is the title layer which appears at the top of each section.  Note the
  it calls the ORLayer constructor setting top and bottom margins and setting
  the bDontDelete flag to TRUE.  The "bDontDelete" flag will cause this layer
  to reprint on each page until all of this layer's sibling layers finish
  printing.
-----------------------------------------------------------------------------*/
ORLayer_ATitle::ORLayer_ATitle( ORLayer *pL, char *psTitle)
               :ORLayer( pL, 12, 12, TRUE)
  {
  strcpy( sTitle, psTitle);
  }

long ORLayer_ATitle::InstantiateChildren( )
  {
  //---- this text element is given an id of '1' so that
  //----   it can be found by it's parent
  new ORText( this, 1, sTitle, IDFntA_Arial14, ORPOS_CENTER);

  return 0;
  }

long ORLayer_ATitle::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    pDC->SelectStockPen( BLACK_PEN);
    pDC->SelectStockBrush( LTGRAY_BRUSH);
    pDC->RoundRect( Left( ), Top( ORINSIDE),
                    pChildren->Left( ) - pChildren->Height( ), Bottom( ORINSIDE),
                    Height( ) / 2, Height( ) / 2);
    pDC->RoundRect( pChildren->Right( ) + pChildren->Height( ), Top( ORINSIDE),
                    Right( ), Bottom( ORINSIDE),
                    Height( ) / 2, Height( ) / 2);

    //---- append " (continued)" onto the end of the section title
    //----   in case this layer prints again at the top of the next page
    if (bPrinted == FALSE)
      {
      //---- get a pointer to the text string
      ORText *pT = (ORText *)Element( 1);

      strcat( sTitle, " (continued)");
      pT->Text( sTitle);
      }
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_AcctSumm::ORLayer_AcctSumm( ORReport *pRep, Account *pAcct_)
                 :ORLayer( pRep)
  {
  pAcct = pAcct_;

  //---- for debugging purposes only
  StringId( "ORLayer_AcctSumm");
  }

long ORLayer_AcctSumm::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    //---- get pointer to second child layer (the column header Layer)
    ORLayer *pL = (ORLayer *)pChildren->Next( );

    pDC->SelectStockPen( BLACK_PEN);

    pDC->MoveToEx( Column( IDColA_TaxInc)->Left( ORINSIDE),     pL->Bottom( ));
    pDC->LineTo( Column( IDColA_NontaxInc)->Right( ORINSIDE), pL->Bottom( ));

    pDC->MoveToEx( Column( IDColA_CostBasis)->Left( ORINSIDE), pL->Bottom( ));
    pDC->LineTo( Column( IDColA_GLUnreal)->Right( ORINSIDE), pL->Bottom( ));
    }

  return lErr;
  }

long ORLayer_AcctSumm::InstantiateChildren( )
  {
  long lErr = 0;

  new ORLayer_ATitle( this, "Account Summary");

  ORLayer *pL = new ORLayer( this);
  new ORText( pL, 0, "Cost\nBasis", IDFntA_Arial10B, ORJUST_CENTER, IDColA_CostBasis);
  new ORText( pL, 0, "Current\nValue", IDFntA_Arial10B, ORJUST_CENTER, IDColA_CurrValue);
  new ORText( pL, 0, "Gain/Loss", IDFntA_Arial10B, ORJUST_CENTER | ORPOS_TOP, IDColA_GLMid);
  new ORText( pL, 0, "Realized", IDFntA_Arial10B, ORJUST_CENTER, IDColA_GLReal);
  new ORText( pL, 0, "Unrealized", IDFntA_Arial10B, ORJUST_CENTER, IDColA_GLUnreal);
  new ORText( pL, 0, "Taxable\nIncome", IDFntA_Arial10B, ORJUST_CENTER, IDColA_TaxInc);
  new ORText( pL, 0, "Non-taxable\nIncome", IDFntA_Arial10B, ORJUST_CENTER, IDColA_NontaxInc);

  pL = new ORLayer( this);
  char  s[31];
  DollarValue( s, pAcct->TotalCostBasis( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_CostBasis);
  DollarValue( s, pAcct->TotalCurrValue( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_CurrValue);
  DollarValue( s, pAcct->TotalRealized( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_GLReal);
  DollarValue( s, pAcct->TotalUnrealized( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_GLUnreal);

  new ORText( pL, 0, "YTD", IDFntA_Arial10B, ORPOS_LEFT, IDColA_TaxInc);
  DollarValue( s, pAcct->YTDTaxInc( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_TaxInc);
  DollarValue( s, pAcct->YTDNontaxInc( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_NontaxInc);

  pL = new ORLayer( this);
  new ORText( pL, 0, "Est. 1995", IDFntA_Arial10B, ORPOS_LEFT, IDColA_TaxInc);
  DollarValue( s, pAcct->EstTaxInc( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_TaxInc);
  DollarValue( s, pAcct->EstNontaxInc( ));
  new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_NontaxInc);

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_AcctAlloc::ORLayer_AcctAlloc( ORReport *pRep, Account *pAcct_)
                  :ORLayer( pRep)
  {
  pAcct = pAcct_;

  //---- for debugging purposes only
  StringId( "ORLayer_AcctAlloc");
  }

long ORLayer_AcctAlloc::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    //---- get pointer to second child layer (the column header Layer)
    ORLayer *pL = (ORLayer *)pChildren->Next( );

    ORColumn *pCol = Column( IDColA_Bars);

    //---- iterate thru layers drawing bars for the bar chart
    for (int i=0; pL!=NULL; pL=pL->Next( ),i++)
      {
      if (astInfo[i].dAmount > 0)
        {
        ORRect  r;
        r.lTop    = pL->Top( ORINSIDE);
        r.lBottom = pL->Bottom( ORINSIDE);

        r.lLeft  = pCol->Left( ORINSIDE);
        r.lRight = pCol->Right( ORINSIDE);
        r.lRight = r.lLeft + (long)(r.Width( ) * astInfo[i].dPct / 100.0);

        pDC->SelectStockPen( BLACK_PEN);
        pDC->SelectStockBrush( LTGRAY_BRUSH);

        pDC->Rectangle( r.lLeft, r.lTop, r.lRight, r.lBottom);
        }
      }
    }

  return lErr;
  }

void ORLayer_AcctAlloc::InitializeInfo( )
  {
  int i = 0;

  //---- initialize info structs
  strcpy( astInfo[i].sLabel, "Cash/Money Fund");
  astInfo[i].dAmount = pAcct->TotalCash( );
  i++;

  strcpy( astInfo[i].sLabel, "CD's");
  astInfo[i].dAmount = pAcct->TotalCDs( );
  i++;

  strcpy( astInfo[i].sLabel, "Stocks");
  astInfo[i].dAmount = pAcct->TotalStocks( );
  i++;

  strcpy( astInfo[i].sLabel, "Mutual Funds");
  astInfo[i].dAmount = pAcct->TotalMutualFunds( );
  i++;

  strcpy( astInfo[i].sLabel, "Corporate Bonds");
  astInfo[i].dAmount = pAcct->TotalCorpBonds( );
  i++;

  strcpy( astInfo[i].sLabel, "Government Bonds");
  astInfo[i].dAmount = pAcct->TotalGovBonds( );

  //---- sum total
  double dTotal = 0;
  for (i=0; i<MAXINFOS; i++)
    dTotal += astInfo[i].dAmount;

  //---- calculate average
  for (i=0; i<MAXINFOS; i++)
    astInfo[i].dPct = astInfo[i].dAmount / dTotal * 100.0;
  }

long ORLayer_AcctAlloc::InstantiateChildren( )
  {
  long lErr = 0;

  new ORLayer_ATitle( this, "Account Allocation");

  InitializeInfo( );

  for (int i=0; i<MAXINFOS; i++)
    {
    ORLayer *pL = new ORLayer( this);
    new ORText( pL, 0, astInfo[i].sLabel, IDFntA_Arial10B, ORPOS_LEFT, IDColA_Center);
    char  s[21];
    DollarValue( s, astInfo[i].dAmount);
    new ORText( pL, 0, s, IDFntA_Arial10, ORJUST_RIGHT, IDColA_Amount);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_StockSummary::ORLayer_StockSummary( ORReport *pRep, Account *pAcct_)
                     :ORLayer( pRep)
  {
  pAcct = pAcct_;

  //---- for debugging purposes only
  StringId( "ORLayer_StockSummary");
  }

long ORLayer_StockSummary::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    //---- get pointer to second child layer (the column header Layer)
    ORLayer *pL = (ORLayer *)pChildren->Next( );

    pDC->SelectStockPen( BLACK_PEN);

    pDC->MoveToEx( Column( IDColA_SMFPrchDate)->Left( ORINSIDE),  pL->Bottom( ));
    pDC->LineTo( Column( IDColA_SMFGainLoss)->Right( ORINSIDE), pL->Bottom( ));
    }

  return lErr;
  }

long ORLayer_StockSummary::InstantiateChildren( )
  {
  long lErr = 0;

  //---- if there are no stocks, skip this layer
  if (pAcct->pStocks == NULL)
    return lErr;

  new ORLayer_ATitle( this, "Stock Summary");

  //---- column header layer - this layer is a "don't delete" layer.
  //---- this will cause the header to reprint on subsequent pages
  //----   until the entire StockSummary Layer is complete
  ORLayer *pL = new ORLayer( this, 0, 0, TRUE);

  //---- text elements for the column headers
  new ORText( pL, 0, "Purch\nDate",    IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchDate);
  new ORText( pL, 0, "Name",           IDFntA_Arial10B, ORJUST_LEFT,   IDColA_SMFName);
  new ORText( pL, 0, "Symbol",         IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFSymbol);
  new ORText( pL, 0, "Purch\nQty",     IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchQty);
  new ORText( pL, 0, "Purch\nPrice",   IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchPri);
  new ORText( pL, 0, "Total\nCost",    IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchCost);
  new ORText( pL, 0, "Current\nQty",   IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrQty);
  new ORText( pL, 0, "Current\nPrice", IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrPri);
  new ORText( pL, 0, "Current\nValue", IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrVal);
  new ORText( pL, 0, "Gain\nLoss",     IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFGainLoss);

  //---- for debugging purposes only
  pL->StringId( "StockSummary Column Header");

  //---- create a layer for each Stock to be reported
  for (Stock *pS=pAcct->pStocks; pS!=NULL; pS=pS->Next( ))
    {
    ORLayer *pL = new ORLayer( this);

    new ORText( pL, 0, pS->sPurchDate, IDFntA_Arial10, 0, IDColA_SMFPrchDate);
    new ORText( pL, 0, pS->sName,      IDFntA_Arial10, ORJUST_LEFT, IDColA_SMFName);
    new ORText( pL, 0, pS->sSymbol,    IDFntA_Arial10, ORJUST_LEFT, IDColA_SMFSymbol);

    char  s[21];
    sprintf( s, "%0.2f", pS->dPurchQty);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchQty);
    DollarValue( s, pS->dPurchPrice);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchPri);
    DollarValue( s, (pS->dPurchPrice * pS->dPurchQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchCost);

    sprintf( s, "%0.2f", pS->dCurrQty);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrQty);
    DollarValue( s, pS->dCurrPrice);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrPri);
    DollarValue( s, (pS->dCurrPrice * pS->dCurrQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrVal);

    DollarValue( s, (pS->dCurrPrice * pS->dCurrQty) -
                    (pS->dPurchPrice * pS->dPurchQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFGainLoss);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_MFSummary::ORLayer_MFSummary( ORReport *pRep, Account *pAcct_)
                  :ORLayer( pRep)
  {
  pAcct = pAcct_;

  //---- for debugging purposes only
  StringId( "ORLayer_MFSummary");
  }

long ORLayer_MFSummary::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    //---- get pointer to second child layer (the column header Layer)
    ORLayer *pL = (ORLayer *)pChildren->Next( );

    pDC->SelectStockPen( BLACK_PEN);

    pDC->MoveToEx( Column( IDColA_SMFPrchDate)->Left( ORINSIDE),  pL->Bottom( ));
    pDC->LineTo( Column( IDColA_SMFGainLoss)->Right( ORINSIDE), pL->Bottom( ));
    }

  return lErr;
  }

long ORLayer_MFSummary::InstantiateChildren( )
  {
  long lErr = 0;

  //---- if there are no funds, skip this layer
  if (pAcct->pFunds == NULL)
    return lErr;

  new ORLayer_ATitle( this, "Mutual Fund Summary");

  //---- column header layer - this layer is a "don't delete" layer.
  //---- this will cause the header to reprint on subsequent pages
  //----   until the entire MFSummary Layer is complete
  ORLayer *pL = new ORLayer( this, 0, 0, TRUE);

  //---- text elements for the column headers
  new ORText( pL, 0, "Purch\nDate",    IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchDate);
  new ORText( pL, 0, "Name",           IDFntA_Arial10B, ORJUST_LEFT,   IDColA_SMFName);
  new ORText( pL, 0, "Symbol",         IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFSymbol);
  new ORText( pL, 0, "Purch\nQty",     IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchQty);
  new ORText( pL, 0, "Purch\nPrice",   IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchPri);
  new ORText( pL, 0, "Total\nCost",    IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFPrchCost);
  new ORText( pL, 0, "Current\nQty",   IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrQty);
  new ORText( pL, 0, "Current\nPrice", IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrPri);
  new ORText( pL, 0, "Current\nValue", IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFCurrVal);
  new ORText( pL, 0, "Gain\nLoss",     IDFntA_Arial10B, ORJUST_CENTER, IDColA_SMFGainLoss);

  //---- for debugging purposes only
  pL->StringId( "MFSumm Column Header");

  //---- create a layer for each Mutual Fund to be reported
  for (MutualFund *pMF=pAcct->pFunds; pMF!=NULL; pMF=pMF->Next( ))
    {
    ORLayer *pL = new ORLayer( this);

    new ORText( pL, 0, pMF->sPurchDate, IDFntA_Arial10, 0, IDColA_SMFPrchDate);

    ORText *pT = new ORText( pL, 0, pMF->sName, IDFntA_Arial10, ORJUST_LEFT, IDColA_SMFName);
    pT->SetMaxWidth( Column( IDColA_TSDesc));

    new ORText( pL, 0, pMF->sSymbol,    IDFntA_Arial10, ORJUST_LEFT, IDColA_SMFSymbol);

    char  s[21];
    sprintf( s, "%0.2f", pMF->dPurchQty);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchQty);
    DollarValue( s, pMF->dPurchPrice);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchPri);
    DollarValue( s, (pMF->dPurchPrice * pMF->dPurchQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFPrchCost);

    sprintf( s, "%0.2f", pMF->dCurrQty);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrQty);
    DollarValue( s, pMF->dCurrPrice);
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrPri);
    DollarValue( s, (pMF->dCurrPrice * pMF->dCurrQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFCurrVal);

    DollarValue( s, (pMF->dCurrPrice * pMF->dCurrQty) -
                    (pMF->dPurchPrice * pMF->dPurchQty));
    new ORText( pL, 0, s, IDFntA_Arial10, 0, IDColA_SMFGainLoss);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_TransSumm::ORLayer_TransSumm( ORReport *pRep, Account *pAcct_)
                  :ORLayer( pRep)
  {
  pAcct = pAcct_;

  //---- for debugging purposes only
  StringId( "ORLayer_TransSumm");
  }

long ORLayer_TransSumm::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    //---- get pointer to second child layer (the column header Layer)
    ORLayer *pL = (ORLayer *)pChildren->Next( );

    pDC->SelectStockPen( BLACK_PEN);

    pDC->MoveToEx( Column( IDColA_TSDate)->Left( ORINSIDE),   pL->Bottom( ));
    pDC->LineTo( Column( IDColA_TSBalance)->Right( ORINSIDE), pL->Bottom( ));
    }

  return lErr;
  }

long ORLayer_TransSumm::InstantiateChildren( )
  {
  long lErr = 0;

  //---- if there are no funds, skip this layer
  if (pAcct->pTransacts == NULL)
    return lErr;

  new ORLayer_ATitle( this, "Transaction Summary");

  //---- column header layer - this layer is a "don't delete" layer.
  //---- this will cause the header to reprint on subsequent pages
  //----   until the entire TransSumm Layer is complete
  ORLayer *pL = new ORLayer( this, 0, 0, TRUE);

  //---- text elements for the column headers
  new ORText( pL, 0, "Trans\nDate", IDFntA_Arial10B, ORJUST_CENTER, IDColA_TSDate);
  new ORText( pL, 0, "Description", IDFntA_Arial10B, ORJUST_LEFT,   IDColA_TSDesc);
  new ORText( pL, 0, "Qty",         IDFntA_Arial10B, ORJUST_CENTER, IDColA_TSShares);
  new ORText( pL, 0, "Price",       IDFntA_Arial10B, ORJUST_CENTER, IDColA_TSPrice);
  new ORText( pL, 0, "Total",       IDFntA_Arial10B, ORJUST_CENTER, IDColA_TSCredDeb);
  new ORText( pL, 0, "Balance",     IDFntA_Arial10B, ORJUST_CENTER, IDColA_TSBalance);

  //---- for debugging purposes only
  pL->StringId( "TransSumm Column Header");

  //---- find first and last for bolding
  Transaction *pFirst = pAcct->pTransacts;
  Transaction *pLast;
  for (Transaction *pT=pAcct->pTransacts; pT!=NULL; pT=pT->Next( ))
    if (pT->Next( ) == NULL)
      pLast = pT;

  //---- create a layer for each Transaction Record to be reported
  for (pT=pAcct->pTransacts; pT!=NULL; pT=pT->Next( ))
    {
    long lFntId = ((pT == pFirst) || (pT == pLast)) ? IDFntA_Arial10B
                                                    : IDFntA_Arial10;

    ORLayer *pL = new ORLayer( this);

    new ORText( pL, 0, pT->sDate, lFntId, ORPOS_TOP, IDColA_TSDate);
    ORText *pORT = new ORText( pL, 0, pT->sDesc, lFntId, ORJUST_LEFT, IDColA_TSDesc);
    pORT->SetMaxWidth( Column( IDColA_TSDesc));

    char  s[21];

    if (pT->dQty != 0)
      {
      sprintf( s, "%0.2f", pT->dQty);
      new ORText( pL, 0, s, lFntId, ORPOS_TOP, IDColA_TSShares);
      DollarValue( s, pT->dPrice);
      new ORText( pL, 0, s, lFntId, ORPOS_TOP, IDColA_TSPrice);
      }

    if (pT->dAmount != 0)
      {
      DollarValue( s, pT->dAmount);
      new ORText( pL, 0, s, lFntId, ORPOS_TOP, IDColA_TSCredDeb);
      }

    if ((pT->dBalance != 0) || (pT == pFirst) || (pT == pLast))
      {
      DollarValue( s, pT->dBalance);
      new ORText( pL, 0, s, lFntId, ORPOS_TOP, IDColA_TSBalance);
      }
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORReport_Account::ORReport_Account( char *psTitle, ABORTPROC lpfnAbortProc_,
                                    HWND hDlgPrint_)
                 :ORReport( psTitle, lpfnAbortProc_)
  {
  pAcct = NULL;

  hDlgPrint = hDlgPrint_;
  }

ORReport_Account::~ORReport_Account( )
  {
  if (pAcct)
    delete pAcct;
  }

long ORReport_Account::Initialize( )
  {
  SendMessage( hDlgPrint, WMU_TITLE, 0, (LPARAM)sTitle);

  long lErr = ORReport::Initialize( );

  if (!lErr)
    pDC->SetBkMode( TRANSPARENT);

  pAcct = new Account( FILE_ACCOUNT_DATA);
  if (pAcct == NULL)
    lErr = APPERR_MEMORY;

  return lErr;
  }

long ORReport_Account::Instantiate( ORPrintDC *pPrintDC)
  {
  long lErr = 0;

  //---- verify data file exists
  FILE *pF = fopen( FILE_ACCOUNT_DATA, "rt");
  if (pF == NULL)
    {
    char  s[101];
    sprintf( s, "Required data file '%s' does not exist.", FILE_ACCOUNT_DATA);
    MessageBox( hDlgPrint, s, NULL, MB_ICONSTOP);
    lErr = APPERR_FILE;
    }

  if (pF)
    fclose( pF);

  if (!lErr)
    lErr = ORReport::Instantiate( pPrintDC);

  return lErr;
  }

long ORReport_Account::InstantiateColumns( )
  {
  //---- columns for the Account Summary layer
  new ORColumn( this, IDColA_CostBasis, "$123,456,789.00", IDFntA_Arial10, 0, 0, 0, ORPOS_LEFT);
  new ORColumn( this, IDColA_CurrValue, "$123,456,789.00", IDFntA_Arial10, 8, 0, IDColA_CostBasis, ORPOS_RIGHT);
  new ORColumn( this, IDColA_GLReal,    "$123,456,789.00", IDFntA_Arial10, 4, 0, IDColA_CurrValue, ORPOS_RIGHT);
  new ORColumn( this, IDColA_GLMid,     "",                IDFntA_Arial10, 4, 0, IDColA_GLReal, ORPOS_RIGHT);
  new ORColumn( this, IDColA_GLUnreal,  "$123,456,789.00", IDFntA_Arial10, 8, 0, IDColA_GLMid, ORPOS_RIGHT);

  new ORColumn( this, IDColA_NontaxInc, "$123,456,789.00", IDFntA_Arial10, 0, 30, 0, ORPOS_RIGHT);
  new ORColumn( this, IDColA_TaxInc,    "$123,456,789.00", IDFntA_Arial10, 0, 8, IDColA_NontaxInc, ORPOS_LEFT);

  //---- columns for the Account Allocation layer
  new ORColumn( this, IDColA_Center,          NULL, IDFntA_Arial10, 0, 0, 0, ORPOS_CENTER);
  new ORColumn( this, IDColA_Amount, "$123,456.00", IDFntA_Arial10, 8, 4, IDColA_Center, ORPOS_RIGHT);
  new ORColumn( this, IDColA_Bars,      "XXXXXXXX", IDFntA_Arial10, 4, 8, IDColA_Amount, ORPOS_RIGHT);

  //---- columns for the Stock Summary and Mutual Fund Summary layers
  new ORColumn( this, IDColA_SMFPrchDate, "12/00/00", IDFntA_Arial10, 4, 4, 0, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFName, "XXXXXXXXXXXXXXXXXXXXXXXXXXX", IDFntA_Arial10, 4, 0, IDColA_SMFPrchDate, ORPOS_RIGHT);

  new ORColumn( this, IDColA_SMFGainLoss, "$12,345.00", IDFntA_Arial10, 4, 4, 0, ORPOS_RIGHT);
  new ORColumn( this, IDColA_SMFCurrVal,  "$12,345.00", IDFntA_Arial10, 4, 4, IDColA_SMFGainLoss, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFCurrPri,     "$123.00", IDFntA_Arial10, 4, 4, IDColA_SMFCurrVal, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFCurrQty,     "999,999", IDFntA_Arial10, 4, 4, IDColA_SMFCurrPri, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFPrchCost, "$12,345.00", IDFntA_Arial10, 4, 4, IDColA_SMFCurrQty, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFPrchPri,     "$123.00", IDFntA_Arial10, 4, 4, IDColA_SMFPrchCost, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFPrchQty,     "999,999", IDFntA_Arial10, 4, 4, IDColA_SMFPrchPri, ORPOS_LEFT);
  new ORColumn( this, IDColA_SMFSymbol,       "XXXXXX", IDFntA_Arial10, 4, 4, IDColA_SMFPrchQty, ORPOS_LEFT);

  //---- columns for the Transaction Summary layers
  new ORColumn( this, IDColA_TSDate, "12/99", IDFntA_Arial10, 24, 4, 0, ORPOS_LEFT);
  new ORColumn( this, IDColA_TSDesc, "XXXXXXXXXXXXXXXXXXXXXXXXXXX", IDFntA_Arial10, 4, 0, IDColA_TSDate, ORPOS_RIGHT);

  new ORColumn( this, IDColA_TSBalance, "$123,456,789.00", IDFntA_Arial10, 4, 24, 0, ORPOS_RIGHT);
  new ORColumn( this, IDColA_TSCredDeb,     "$123,456.00", IDFntA_Arial10, 4, 4, IDColA_TSBalance, ORPOS_LEFT);
  new ORColumn( this, IDColA_TSPrice,         "$1,234.00", IDFntA_Arial10, 4, 4, IDColA_TSCredDeb, ORPOS_LEFT);
  new ORColumn( this, IDColA_TSShares,          "999,999", IDFntA_Arial10, 4, 4, IDColA_TSPrice, ORPOS_LEFT);

  return 0;
  }

/*-----------------------------------------------------------------------------
  these long, detailed calls to create ORFonts are not always required.  they
    are used here because the ORDemo.Exe program can be run on a variety of
    different machines with different fonts available.  by providing the
    Windows font mapper with the maximum amount of information, it will be
    able to provide a similar font if the desired font is not available.
    if you know ahead of time what fonts will be available when your reports
    are printed, you can simply create your fonts in the
    ORReport::InstantiateFonts member function as follows:
      new ORFont( this, IDFntA_Arial10,  "Arial", 10.0);
      new ORFont( this, IDFntA_Arial10B, "Arial", 10.0, 0, 0, 0, FW_BLACK);
-----------------------------------------------------------------------------*/
long ORReport_Account::InstantiateFonts( )
  {
  new ORFont( this, IDFntA_Arial10, "Arial", 10.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntA_Arial10B, "Arial", 10.0, 0.0, 0, 0, FW_BLACK,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntA_Arial12, "Arial", 12.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntA_Arial14, "Arial", 14.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntA_Arial18, "Arial", 18.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntA_Arial20, "Arial", 20.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));

  return 0;
  }

long ORReport_Account::InstantiateChildren( )
  {
  long lErr = 0;

  if (!lErr)
    {
    ORLayer_AcctSumm *pL = new ORLayer_AcctSumm( this, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  if (!lErr)
    {
    ORLayer_AcctAlloc *pL = new ORLayer_AcctAlloc( this, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  if (!lErr)
    {
    ORLayer_StockSummary *pL = new ORLayer_StockSummary( this, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  if (!lErr)
    {
    ORLayer_MFSummary *pL = new ORLayer_MFSummary( this, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  if (!lErr)
    {
    ORLayer_TransSumm *pL = new ORLayer_TransSumm( this, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  return lErr;
  }

long ORReport_Account::InstantiatePageLayers( )
  {
  long lErr = 0;

  //---- add header page layer
  if (!lErr)
    {
    ORLayer_AHeader *pL = new ORLayer_AHeader( this, ORPOS_TOP, pAcct);
    if (pL)
      lErr = pL->Instantiate( );
    }

  //---- no footer

  return lErr;
  }

long ORReport_Account::PrintPage( )
  {
  char  s[51];
  sprintf( s, "Printing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  long lErr = ORReport::PrintPage( );

  sprintf( s, "Initializing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  return lErr;
  }
