/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#include "orcc.h"
#include <windows.h>
#pragma  hdrstop

#include <string.h>
#include "oracct.h"
#include "ordemo.h"
#include "orpackls.h"
#include "orprev.h"
#include "orprntdc.h"
#include "orsales.h"
#include "orsource.h"
#include "ortulayr.h"


ORBool        bUserAbort;
ORBool        bPreview;
ORBool        bPrevHelpDisplayed;
HINSTANCE     hInst;
HWND          hWnd;
HWND          hDlgPrint;
ORPreviewWnd *pWndPrev;
short         shCurrPreviewMode;


//-----------------------------------------------------------------------------
//-- The next 2 functions are used for the status/abort dialog.  They are
//-- standard Windows stuff.
//--
BOOL CALLBACK ORDEMOEXPORT AbortProc( HDC, short)
  {
  MSG  stMsg;

  while (!bUserAbort && (PeekMessage( &stMsg, NULL, 0, 0, PM_REMOVE)))
    {
    if (!hDlgPrint || IsDialogMessage( hDlgPrint, &stMsg))
      {
      TranslateMessage( &stMsg);
      DispatchMessage( &stMsg);
      }
    }

  return !bUserAbort;
  }

BOOL CALLBACK ORDEMOEXPORT PrintDlgProc( HWND hDlg, WORD wMessage,
                                         WPARAM, LPARAM l)
  {
  if (hDlgPrint)          //-- only process if Abort dialog
    {
    switch (wMessage)
      {
      case WM_INITDIALOG :
          EnableMenuItem( GetSystemMenu( hDlg, FALSE), SC_CLOSE, MF_GRAYED);
          return TRUE;
      case WM_COMMAND :
          bUserAbort = TRUE;
          EnableWindow( GetParent( hDlg), TRUE);
          DestroyWindow( hDlg);
          hDlgPrint = 0;
          return TRUE;
      case WMU_TITLE :
          SetDlgItemText( hDlg, IDC_PSTitle, (LPSTR)l);
          return TRUE;
      case WMU_PRINTINGPAGE :
          SetDlgItemText( hDlg, IDC_PSStatus, (LPSTR)l);
          return TRUE;
      }
    }

  return FALSE;
  }

//-----------------------------------------------------------------------------
//-- This function creates the Object Reports Report Preview Window and tells
//-- the Preview Window to open the Preview File.
//--
void Preview( )
  {
  //---- if the window does not exist, create a new one
  if (pWndPrev == NULL)
    {
    pWndPrev = new ORPreviewWnd( hWnd);

    switch (shCurrPreviewMode)
      {
      case IDM_ModeFullPage         : pWndPrev->SetDisplayMode( ORDISP_FULLPAGE);          break;
      case IDM_ModeScreenPageWidth  : pWndPrev->SetDisplayMode( ORDISP_SCREENPAGEWIDTH);   break;
      case IDM_ModeScreenPageHeight : pWndPrev->SetDisplayMode( ORDISP_SCREENPAGEHEIGHT);  break;
      case IDM_ModeWindowPageWidth  : pWndPrev->SetDisplayMode( ORDISP_WINDOWPAGEWIDTH);   break;
      case IDM_ModeWindowPageHeight : pWndPrev->SetDisplayMode( ORDISP_WINDOWPAGEHEIGHT);  break;
      }
    }
  else
    pWndPrev->Close( );  //-- Preview Window already exists, just close Preview

  //---- window now exists, open the Preview File
  long lErr = pWndPrev->Open( "ObjReps.Pre");
  if (lErr)
    {
    delete pWndPrev;     //-- if an error occurs, delete the Preview Window
    pWndPrev = NULL;

    MessageBox( hWnd, "Error opening preview file.", NULL, MB_ICONSTOP);
    }
  }

//-----------------------------------------------------------------------------
//-- Responds to a menu request to change the current Preview Mode.
//--
void ChangePreviewMode( WPARAM wParam)
  {
  //---- check and uncheck the appropriate menu items
  CheckMenuItem( GetMenu( hWnd), shCurrPreviewMode, MF_UNCHECKED | MF_BYCOMMAND);
  CheckMenuItem( GetMenu( hWnd), wParam, MF_CHECKED | MF_BYCOMMAND);

  //---- tell the Preview Window what the new Preview Mode is.
  switch (wParam)
    {
    case IDM_ModeFullPage :
      if (pWndPrev)
        pWndPrev->SetDisplayMode( ORDISP_FULLPAGE);
      break;
    case IDM_ModeScreenPageWidth :
      if (pWndPrev)
        pWndPrev->SetDisplayMode( ORDISP_SCREENPAGEWIDTH);
      break;
    case IDM_ModeScreenPageHeight :
      if (pWndPrev)
        pWndPrev->SetDisplayMode( ORDISP_SCREENPAGEHEIGHT);
      break;
    case IDM_ModeWindowPageWidth :
      if (pWndPrev)
        pWndPrev->SetDisplayMode( ORDISP_WINDOWPAGEWIDTH);
      break;
    case IDM_ModeWindowPageHeight :
      if (pWndPrev)
        pWndPrev->SetDisplayMode( ORDISP_WINDOWPAGEHEIGHT);
      break;
    }

  //---- save new Preview Mode
  shCurrPreviewMode = (short)wParam;
  }

//-----------------------------------------------------------------------------
//-- Simply displays a dialog which lists the keys available during preview.
//--
void DisplayPreviewHelp( )
  {
  //---- display the help dialog box only once
  if ((pWndPrev) && (bPrevHelpDisplayed == FALSE))
    {
    //---- create dialog box procedure
    DLGPROC lpProcDlg = (DLGPROC)MakeProcInstance( (FARPROC)PrintDlgProc,
                                                   hInst);

    //---- display the dialog
    DialogBox( hInst, "DLG_PrevHelp", hWnd, (DLGPROC)lpProcDlg);

#if !defined(__WIN32__) && !defined(_WIN32)
    //---- cleanup
    if (lpProcDlg)
      FreeProcInstance( (FARPROC)lpProcDlg);
#endif

    //---- set flag to prevent this dialog from being displayed again
    bPrevHelpDisplayed = TRUE;
    }
  }

//-----------------------------------------------------------------------------
//-- Creates a specified Report based on a selected menu item.
//--
void GenerateReport( WPARAM wParam)
  {
  UpdateWindow( hWnd);

  bUserAbort = FALSE;
  hDlgPrint  = 0;

  //---- create a Windows Dialog Procedure
  DLGPROC lpProcDlgPrint = (DLGPROC)MakeProcInstance( (FARPROC)PrintDlgProc,
                                                      hInst);
  //---- create a Windows Dialog
  if (lpProcDlgPrint)
    hDlgPrint = CreateDialog( hInst, "DLG_PrintStatus", hWnd, lpProcDlgPrint);

  //---- create a Windows Abort Procedure
  ABORTPROC lpProcAbort = (ABORTPROC)MakeProcInstance( (FARPROC)AbortProc,
                                                       hInst);

  //---- create an ORPrintDC for the default Windows Printer
  ORPrintDC *pPrintDC = ORPrintDC::GetDefaultPrinterDC( );
  if (pPrintDC == NULL)
    MessageBox( NULL, "Failed to create printer DC.", NULL, MB_ICONSTOP);

  //---- if there's an existing Preview Window, close the current Preview File
  if (pWndPrev)
    pWndPrev->Close( );

  //---- if everything succeeded, create the ORReport object
  if (pPrintDC && lpProcAbort && lpProcDlgPrint)
    {
    //---- disable the main window while using Abort/Status Dialog
    EnableWindow( hWnd, FALSE);

    ORReport *pRep = NULL;

    //---- select the specified ORReport object
    switch (wParam)
      {
      case IDM_RepTutorLayer :
        pRep = new ORReport_LayerTutor( "Layer Tutorial",
                                        lpProcAbort, hDlgPrint);
        break;
      case IDM_RepSales :
        pRep = new ORReport_Sales( "Sales by Month by Product",
                                   lpProcAbort, hDlgPrint);
        break;
      case IDM_RepPackingList :
        pRep = new ORReport_PackList( "Packing List Demo",
                                      lpProcAbort, hDlgPrint);
        break;
      case IDM_RepAccount :
        pRep = new ORReport_Account( "Account Demo",
                                     lpProcAbort, hDlgPrint);
        break;
      case IDM_SrcTutorLayer :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORTuLayr.CPP");
        break;
      case IDM_SrcSales :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORSales.CPP");
        break;
      case IDM_SrcPackingList :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORPackLs.CPP");
        break;
      case IDM_SrcAccount :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORAcct.CPP");
        break;
      case IDM_SrcAnnual :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORAnnual.CPP");
        break;
      case IDM_SrcSource :
        pRep = new ORReport_Source( "Source Code", lpProcAbort,
                                    hDlgPrint, "ORSource.CPP");
        break;
      }

    if (pRep)
      {
      //---- if output is going to Preview Window,
      //----   tell the ORPrintDC to send the out to a Preview File
      if (bPreview)
        pPrintDC->SetPreviewFile( "ObjReps.Pre");

      //---- generate the report
      pRep->Instantiate( pPrintDC);
      delete pRep;

      //---- if output was going to the printer, it's already there
      //---- if output was going to a Preview File, create a Preview Window
      if (bPreview)
        Preview( );
      }
    }

  //---- if the user did not abort, clean up the Abort/Status dialog
  if (!bUserAbort)
    {
    EnableWindow( hWnd, TRUE);
    DestroyWindow( hDlgPrint);
    }

  //---- don't need ORPrinterDC anymore
  if (pPrintDC)
    delete pPrintDC;

  //---- more cleanup
#if !defined(__WIN32__) && !defined(_WIN32)
  if (lpProcAbort)
    FreeProcInstance( (FARPROC)lpProcAbort);
  if (lpProcDlgPrint)
    FreeProcInstance( (FARPROC)lpProcDlgPrint);
#endif

  DisplayPreviewHelp( );
  }

//-----------------------------------------------------------------------------
//-- This function prints the Report which is currently being Previewed.
//--
void PrintThePreview( )
  {
  //---- if the is not a Report being Previewed, we can't print it
  if ((pWndPrev == NULL) || (pWndPrev->IsOpen( ) == FALSE))
    {
    MessageBox( hWnd, "You have not previewed a report.", "Ooops!",
                MB_ICONINFORMATION);
    return;
    }

  bUserAbort = FALSE;
  hDlgPrint  = 0;

  //---- create a Windows Dialog Procedure
  DLGPROC lpProcDlgPrint = (DLGPROC)MakeProcInstance( (FARPROC)PrintDlgProc,
                                                      hInst);
  //---- create a Windows Dialog
  if (lpProcDlgPrint)
    hDlgPrint = CreateDialog( hInst, "DLG_PrintStatus", hWnd, lpProcDlgPrint);

  //---- create a Windows Abort Procedure
  ABORTPROC lpProcAbort = (ABORTPROC)MakeProcInstance( (FARPROC)AbortProc,
                                                       hInst);

  //---- create an ORPrintDC for the default Windows Printer
  ORPrintDC *pPrintDC = ORPrintDC::GetDefaultPrinterDC( );
  if (pPrintDC == NULL)
    MessageBox( NULL, "Failed to create printer DC.", NULL, MB_ICONSTOP);

  //---- if everything succeeded
  if (pPrintDC && lpProcAbort && lpProcDlgPrint)
    {
    //---- disable the main window while using Abort/Status Dialog
    EnableWindow( hWnd, FALSE);

    //---- tell Preview Window to print the Report currently being Previewed
    pWndPrev->Print( pPrintDC);
    }

  //---- if the user did not abort, clean up the Abort/Status dialog
  if (!bUserAbort)
    {
    EnableWindow( hWnd, TRUE);
    DestroyWindow( hDlgPrint);
    }

  //---- don't need ORPrinterDC anymore
  if (pPrintDC)
    delete pPrintDC;

  //---- more cleanup
#if !defined(__WIN32__) && !defined(_WIN32)
  if (lpProcAbort)
    FreeProcInstance( (FARPROC)lpProcAbort);
  if (lpProcDlgPrint)
    FreeProcInstance( (FARPROC)lpProcDlgPrint);
#endif
  }

//-----------------------------------------------------------------------------
//-- Handles menu item commands
//--
int WMCommand( WPARAM wParam)
  {
  switch (wParam)
    {
    //-------- print the currently displayed Report Preview
    case IDM_FilePrint :
      PrintThePreview( );
      break;

    //-------- exit the program
    case IDM_FileExit :
      if (pWndPrev)
        {
        delete pWndPrev;
        pWndPrev = NULL;
        }
      PostQuitMessage( 0);
      break;

    //-------- change output destination from preview to printer
    case IDM_OutputToPrinter :
      if (bPreview == TRUE)
        {
        //---- delete the preview window
        if (pWndPrev)
          {
          delete pWndPrev;
          pWndPrev = NULL;
          }

        unlink( "ObjReps.Pre");

        //---- set demo application stuff
        bPreview = FALSE;
        CheckMenuItem( GetMenu( hWnd), IDM_OutputToPrinter, MF_CHECKED | MF_BYCOMMAND);
        CheckMenuItem( GetMenu( hWnd), IDM_OutputToPreview, MF_UNCHECKED | MF_BYCOMMAND);
        }
      break;

    //-------- change output destination from printer to preview
    case IDM_OutputToPreview :
      if (bPreview == FALSE)
        {
        //---- set demo application info
        bPreview = TRUE;
        CheckMenuItem( GetMenu( hWnd), IDM_OutputToPrinter, MF_UNCHECKED | MF_BYCOMMAND);
        CheckMenuItem( GetMenu( hWnd), IDM_OutputToPreview, MF_CHECKED | MF_BYCOMMAND);
        }
      break;

    //-------- change preview display mode
    case IDM_ModeFullPage :
    case IDM_ModeScreenPageWidth :
    case IDM_ModeScreenPageHeight :
    case IDM_ModeWindowPageWidth :
    case IDM_ModeWindowPageHeight :
      ChangePreviewMode( wParam);
      break;

    default :
      GenerateReport( wParam);
      break;
    }

  return 0;
  }

//-----------------------------------------------------------------------------
//-- Main Windows procedure for the main window.
//--
long CALLBACK ORDEMOEXPORT WndProc( HWND hWnd, UINT wMsg,
                                    WPARAM wParam, LPARAM lParam)
  {
  switch (wMsg)
    {
    case WM_COMMAND :
      return WMCommand( wParam);        //-- menu items

    case WM_HSCROLL :      //-- handles horizontal scrolling
      if (pWndPrev)
        {
        int nPos = pWndPrev->GetNewScrollPos( hWnd, SB_HORZ, wParam, lParam);
        SetScrollPos( hWnd, SB_HORZ, nPos, TRUE);
        pWndPrev->ViewX( nPos);
        }
      return 0;

    case WM_VSCROLL :      //-- handles vertical scrolling
      if (pWndPrev)
        {
        int nPos = pWndPrev->GetNewScrollPos( hWnd, SB_VERT, wParam, lParam);
        SetScrollPos( hWnd, SB_VERT, nPos, TRUE);
        pWndPrev->ViewY( nPos);
        }
      return 0;

    case WM_KEYDOWN :      //-- keystrokes to manipulate the preview
      if (pWndPrev)
        {
        switch ((int)wParam)
          {
          case VK_PRIOR  : pWndPrev->PagePrev( );      break;
          case VK_NEXT   : pWndPrev->PageNext( );      break;
          case VK_DELETE : pWndPrev->Invalidate( );    break;
          }
        }
      break;

    case WM_CHAR :         //-- keystrokes to manipulate the preview
      if (pWndPrev)
        {
        switch ((int)wParam)
          {
          case '+' :
              if (pWndPrev->Zoom( ) < 400)
                pWndPrev->Zoom( (short)(pWndPrev->Zoom( ) * 2));
              break;
          case '-' :
              if (pWndPrev->Zoom( ) > 25)
                pWndPrev->Zoom( (short)(pWndPrev->Zoom( ) / 2));
              break;
          }
        }
      break;

    case WM_PAINT :        //-- just pass update messages to the preview window
      if (pWndPrev)        //--   it handles them automatically
        {
        long lErr = pWndPrev->Paint( );
        if (lErr)
          {
          delete pWndPrev;
          pWndPrev = NULL;

          MessageBox( hWnd, "Error painting preview.", NULL, MB_ICONSTOP);
          return 0;
          }

        SetScrollRange( hWnd, SB_HORZ, 0, pWndPrev->ScrollMaxX( ), TRUE);
        SetScrollRange( hWnd, SB_VERT, 0, pWndPrev->ScrollMaxY( ), TRUE);

        return 0;
        }
      break;

    case WM_DESTROY :
      if (pWndPrev)
        {
        delete pWndPrev;
        pWndPrev = NULL;
        }

      PostQuitMessage( 0);
      return 0;
    }

  return DefWindowProc( hWnd, wMsg, wParam, lParam);
  }


//-----------------------------------------------------------------------------
//-- Entry point.
//--
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR, int)
  {
  char *psAppName = "ORDemo";

  //---- register window class on first instance
  if (!hPrevInstance)
    {
    WNDCLASS  stWndClass;

    stWndClass.style         = CS_HREDRAW | CS_VREDRAW;
    stWndClass.lpfnWndProc   = WndProc;
    stWndClass.cbClsExtra    = 0;
    stWndClass.cbWndExtra    = 0;
    stWndClass.hInstance     = hInstance;
    stWndClass.hIcon         = LoadIcon( NULL, IDI_APPLICATION);
    stWndClass.hCursor       = LoadCursor( NULL, IDC_ARROW);
    stWndClass.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH);
    stWndClass.lpszMenuName  = "ORDemoMenu";
    stWndClass.lpszClassName = "ORDemo";

    RegisterClass( &stWndClass);
    }

  //---- create application's main window
  hWnd = CreateWindow( "ORDemo", "Object Reports Demo",
                       WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       NULL, NULL, hInstance, NULL);
  if (hWnd == 0)
    return 0;

  hInst = hInstance;

  bPreview = TRUE;
  pWndPrev = NULL;
  bPrevHelpDisplayed = FALSE;

  //---- setup menu items
  CheckMenuItem( GetMenu( hWnd), IDM_OutputToPreview, MF_CHECKED | MF_BYCOMMAND);
  CheckMenuItem( GetMenu( hWnd), IDM_ModeScreenPageWidth, MF_CHECKED | MF_BYCOMMAND);

  shCurrPreviewMode = IDM_ModeScreenPageWidth;

  ShowWindow( hWnd, SW_SHOWMAXIMIZED);
  UpdateWindow( hWnd);

  MSG  stMsg;

  while (GetMessage( &stMsg, NULL, 0, 0))
    {
    TranslateMessage( &stMsg);
    DispatchMessage( &stMsg);
    }

  //---- delete the Preview File
  unlink( "ObjReps.Pre");

  UnregisterClass( psAppName, hInstance);

  return stMsg.wParam;
  }
