/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/


#ifndef  _OROBJECT_H
#define  _OROBJECT_H


#include "oreports.h"
#include "nodedblk.h"


#define  ORPOS_LEFT     0x00000001
#define  ORPOS_CENTER   0x00000002
#define  ORPOS_RIGHT    0x00000004
#define  ORPOS_ALLH     (ORPOS_LEFT | ORPOS_CENTER | ORPOS_RIGHT)
#define  ORPOS_TOP      0x00000008
#define  ORPOS_MIDDLE   0x00000010
#define  ORPOS_BOTTOM   0x00000020
#define  ORPOS_ALLV     (ORPOS_TOP | ORPOS_MIDDLE | ORPOS_BOTTOM)
#define  ORPOS_ALL      (ORPOS_ALLH | ORPOS_ALLV)

#define  ORJUST_LEFT    0x00000040
#define  ORJUST_CENTER  0x00000080
#define  ORJUST_RIGHT   0x00000100
#define  ORJUST_ALLH    (ORJUST_LEFT | ORJUST_CENTER | ORJUST_RIGHT)
#define  ORJUST_TOP     0x00000200
#define  ORJUST_MIDDLE  0x00000400
#define  ORJUST_BOTTOM  0x00000800
#define  ORJUST_ALLV    (ORJUST_TOP | ORJUST_MIDDLE | ORJUST_BOTTOM)
#define  ORJUST_ALL     (ORJUST_ALLH | ORJUST_ALLV)


#define  TYPE_OROBJECT           1
#define  TYPE_ORELEMENT          2
#define  TYPE_ORTEXT             3
#define  TYPE_ORLAYER            4
#define  TYPE_ORCOLUMN           5
#define  TYPE_ORFONT             6
#define  TYPE_ORPEN              7
#define  TYPE_ORBRUSH            8
#define  TYPE_ORDC               9
#define  TYPE_ORPREVIEWDC       10
#define  TYPE_ORPRINTDC         11


#define  OR_ERRORBASE               -100
#define  ORERR_ERROR                (OR_ERRORBASE - 0)
#define  ORERR_MEMORY               (OR_ERRORBASE - 1)
#define  ORERR_FILE                 (OR_ERRORBASE - 2)
#define  ORERR_ENDOFFILE            (OR_ERRORBASE - 3)
#define  ORERR_BADFONT              (OR_ERRORBASE - 4)
#define  ORERR_DC                   (OR_ERRORBASE - 5)
#define  ORERR_USERABORT            (OR_ERRORBASE - 6)
#define  ORERR_BADTYPE              (OR_ERRORBASE - 7)
#define  ORERR_BADRELID             (OR_ERRORBASE - 8)
#define  ORERR_INSTANCENOTCOMPLETE  (OR_ERRORBASE - 9)
#define  ORERR_NOTDRAWN             (OR_ERRORBASE - 10)
#define  ORERR_FILEFORMAT           (OR_ERRORBASE - 11)


#define  ORINSIDE     1
#define  OROUTSIDE    2


class OREXPORT ORDC;


class OREXPORT ORObject : public NodeDoubleLink
  {
  public:
    long    lError;

    ORObject( long lId = 0, long lAttr = 0);
    virtual ~ORObject( );

    virtual void   AddY( long lDY);
    virtual long   Print( ORDC *);
    virtual void   SetX( long lX);
    virtual void   SetY( long lY);
    virtual ORBool TypeOf( long lType);

    virtual long   Attr( )                      { return lAttr; }
    virtual long   Bottom( short = OROUTSIDE)   { return rectPos.lBottom; }
            void   DrawOutline( ORBool b = TRUE) { bDrawOutline = b; }
    virtual long   Height( short = OROUTSIDE)   { return rectPos.Height( ); }
    virtual long   Id( )                        { return lId; }
    virtual long   Left( short = OROUTSIDE)     { return rectPos.lLeft; }
    virtual char  *Name( )                      { return "ORObject"; }
            ORObject *Next( )                   { return (ORObject *)pNext; }
    virtual long   ORJUST( )                 { return (Attr( ) & ORJUST_ALL); }
    virtual long   ORPOS( )                   { return (Attr( ) & ORPOS_ALL); }
    virtual long   Position( ORRect &)          { return 0; }
            ORObject *Prev( )                   { return (ORObject *)pPrev; }
    virtual long   Right( short = OROUTSIDE)    { return rectPos.lRight; }
    virtual long   Size( ORRect &, ORDC *)      { return 0; }
    virtual long   SizeAndPosition( ORRect &, ORDC *)  { return 0; }
    virtual long   Top( short = OROUTSIDE)      { return rectPos.lTop; }
    virtual long   Type( )                      { return TYPE_OROBJECT; }
    virtual long   Width( short = OROUTSIDE)    { return rectPos.Width( ); }

  protected:
    ORBool  bDrawOutline;
    long    lAttr;
    long    lId;
    ORRect  rectPos;

    virtual long   DrawOutline( ORDC *);

  private:
  };


#endif    //---- _OROBJECT_H
