/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#include "orcc.h"
#include <windows.h>
#pragma  hdrstop

#include <stdio.h>
#include <string.h>
#include "orcolumn.h"
#include "ordemo.h"
#include "orfont.h"
#include "orpackls.h"
#include "orpen.h"
#include "orprntdc.h"
#include "ortext.h"
#include "packlist.h"


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_PTitle::ORLayer_PTitle( ORReport_PackList *pRepPackList_)
               :ORLayer( pRepPackList_, 0, 6, 6)
  {
  }

long ORLayer_PTitle::InstantiateChildren( )
  {
  ORLayer *pL = new ORLayer( this);
  new ORText( pL, 0, "Rollins Software Inc", IDFntP_Arial16, ORPOS_CENTER);

  pL = new ORLayer( this);
  new ORText( pL, 0, "PACKING LIST", IDFntP_Arial24, (ORPOS_CENTER | ORJUST_CENTER), IDColP_LeftHalf);
  new ORText( pL, 0, "SALES ORDER",  IDFntP_Arial24, (ORPOS_CENTER | ORJUST_CENTER), IDColP_RightHalf);

  return 0;
  }

long ORLayer_PTitle::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    ORRect  r;
    memcpy( &r, &rectPos, sizeof( r));
    r.lTop    -= MarginTop( );
    r.lBottom += MarginBottom( );

    ORPen *pPen = Pen( IDPenP_2PtsInside);

    pDC->Select( pPen);
    pDC->SelectStockBrush( HOLLOW_BRUSH);
    pDC->RoundRect( r.lLeft, r.lTop, r.lRight, r.lBottom, 60, 60);

    long lW = pPen->Width( );
    pDC->RoundRect( r.lLeft + lW * 2,  r.lTop + lW * 2,
                    r.lRight - lW * 2, r.lBottom - lW * 2);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_BillShip::ORLayer_BillShip( ORReport_PackList *pRepPackList_,
                                    PackingList *pPackList_)
                 :ORLayer( pRepPackList_, 0, 6, 6)
  {
  pPackList = pPackList_;
  }

long ORLayer_BillShip::InstantiateChildren( )
  {
  char  s[201];

  new ORText( this, 1, "SHIP TO: ", IDFntP_Arial12, ORJUST_LEFT | ORPOS_TOP, IDColP_LeftHalf);
  sprintf( s, "Attn: %s\n"
              "%s\n"
              "%s\n"
              "%s, %s  %s",
              pPackList->sShipToName,
              pPackList->sShipToAddr1,
              pPackList->sShipToAddr2,
              pPackList->sShipToCity, pPackList->sShipToState,
                                      pPackList->sShipToZip);
  new ORText( this, 2, s, IDFntP_Arial10, ORPOS_RIGHT, 1);
  new ORText( this, 3, "Phone:\nFax:", IDFntP_Arial10, ORPOS_BOTTOM, 2);
  sprintf( s, " %s\n %s", pPackList->sShipToPhone, pPackList->sShipToFax);
  new ORText( this, 0, s, IDFntP_Arial10, ORPOS_RIGHT, 3);

  ORText *pT = new ORText( this, 5, "BILL TO: ", IDFntP_Arial12, ORJUST_LEFT | ORPOS_TOP, IDColP_RightHalf);
  pT->DrawOutline( );
  sprintf( s, "Attn: %s\n"
              "%s\n"
              "%s\n"
              "%s",
              pPackList->sBillToName,
              pPackList->sBillToAddr1,
              pPackList->sBillToAddr2,
              pPackList->sBillToCity);
  pT = new ORText( this, 6, s, IDFntP_Arial10, ORPOS_RIGHT, 5);
  pT->DrawOutline( );
  pT = new ORText( this, 7, "Relative to:\nfield above:", IDFntP_Arial10, ORPOS_BOTTOM, 6);
  pT->DrawOutline( );
  sprintf( s, " %s\n %s", pPackList->sBillToPhone, pPackList->sBillToFax);
  pT = new ORText( this, 0, s, IDFntP_Arial10, ORPOS_RIGHT, 7);
  pT->DrawOutline( );

  return 0;
  }

long ORLayer_BillShip::SizeAndPosition_Elems_CalcHt( )
  {
  for (ORElement *pE=(ORElement *)pChildren; pE!=NULL; pE=pE->Next( ))
    {
    long lHeightRel = pE->HeightRel( );
    if (lHeightRel > Height( ORINSIDE))
      rectPos.lBottom = rectPos.lTop + lHeightRel;
    }

  return 0;
  }

long ORLayer_BillShip::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    ORRect  r;
    memcpy( &r, &rectPos, sizeof( r));
    r.lTop    -= MarginTop( );
    r.lBottom += MarginBottom( );

    ORPen *pPen = Pen( IDPenP_2PtsInside);

    pDC->Select( pPen);
    pDC->SelectStockBrush( HOLLOW_BRUSH);
    pDC->RoundRect( r.lLeft, r.lTop, r.lRight, r.lBottom, 60, 60);

    long lW = pPen->Width( );
    pDC->RoundRect( r.lLeft + lW * 2,  r.lTop + lW * 2,
                    r.lRight - lW * 2, r.lBottom - lW * 2);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_Spacer::ORLayer_Spacer( ORReport_PackList *pRepPackList_,
                                int nType_, PackingList *pPackList_,
                                int nMarginTop_, int nMarginBottom_)
               :ORLayer( pRepPackList_, 0, nMarginTop_, nMarginBottom_)
  {
  nType     = nType_;
  pPackList = pPackList_;
  }

long ORLayer_Spacer::InstantiateChildren( )
  {
  switch (nType)
    {
    case 1 :
      char  s[31];
      sprintf( s, "%s %d, %d", pPackList->sOrderMonth, pPackList->nOrderDay,
                               pPackList->nOrderYear);
      new ORText( this, 1, "ORDER DATE:  ", IDFntP_Arial16, ORJUST_LEFT, IDColP_LeftHalf);
      new ORText( this, 0, s, IDFntP_Arial16, ORPOS_RIGHT, 1);
      sprintf( s, "%s", pPackList->sTakenBy);
      new ORText( this, 2, "TAKEN BY:  ", IDFntP_Arial16, ORJUST_LEFT, IDColP_RightHalf);
      new ORText( this, 0, s, IDFntP_Arial16, ORPOS_RIGHT, 2);
      break;
    case 2 :
      new ORText( this, 0, "PRODUCT ORDER INFORMATION", IDFntP_Arial24, ORJUST_LEFT, IDColP_LeftHalf);
      break;
    case 3 :
      break;
    }

  return 0;
  }

long ORLayer_Spacer::PrePrint( ORDC *pDC)
  {
  pDC->SelectStockPen( NULL_PEN);
  pDC->SelectStockBrush( LTGRAY_BRUSH);
  pDC->RoundRect( Left( ), Top( ), Right( ), Bottom( ), 45, 45);

  return ORLayer::PrePrint( pDC);
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_Item::ORLayer_Item( ORLayer *pParentLayer_,
                            PackingList *pPackList_, int nItem_)
             :ORLayer( pParentLayer_)
  {
  pPackList = pPackList_;
  nItem = nItem_;
  }

long ORLayer_Item::InstantiateChildren( )
  {
  if (nItem == -1)
    {
    new ORText( this, 0, "ITEM#",       IDFntP_Arial12, ORJUST_CENTER, IDColP_ItemNo);
    new ORText( this, 0, "QTY",         IDFntP_Arial12, ORJUST_CENTER, IDColP_Quantity);
    new ORText( this, 0, "CODE",        IDFntP_Arial12, ORJUST_CENTER, IDColP_Code);
    new ORText( this, 0, "DESCRIPTION", IDFntP_Arial12, ORJUST_CENTER, IDColP_Desc);
    new ORText( this, 0, "UNIT",        IDFntP_Arial12, ORJUST_CENTER, IDColP_UnitPrice);
    new ORText( this, 0, "NOTES",       IDFntP_Arial12, ORJUST_CENTER, IDColP_Notes);
    }
  else if (nItem < pPackList->nItems)
    {
    char  s[31];
    PLItem &plItem = pPackList->aItems[nItem];
    sprintf( s, "%d", (nItem + 1));
    new ORText( this, 0, s, IDFntP_Arial10, ORJUST_CENTER | ORPOS_TOP, IDColP_ItemNo);

    sprintf( s, "%d", plItem.nQty);
    new ORText( this, 0, s, IDFntP_Arial10, ORJUST_CENTER | ORPOS_TOP, IDColP_Quantity);
    new ORText( this, 0, plItem.sCode, IDFntP_Arial10, ORJUST_CENTER | ORPOS_TOP, IDColP_Code);
    new ORText( this, 0, plItem.sDesc, IDFntP_Arial10, ORPOS_TOP | ORJUST_LEFT, IDColP_Desc);

    sprintf( s, "$%2.2f", plItem.dPrice);
    new ORText( this, 0, s, IDFntP_Arial10, ORPOS_TOP, IDColP_UnitPrice);

    ORText *pT = new ORText( this, 0, plItem.sNotes, IDFntP_Arial10, ORJUST_LEFT | ORPOS_TOP, IDColP_Notes);

    pT->SetMaxWidth( Column( IDColP_Notes));
    }
  else
    new ORText( this, 0, "", IDFntP_Arial10, ORJUST_CENTER, IDColP_ItemNo);

  return 0;
  }

long ORLayer_Item::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    ORPen *pPen = Pen( IDPenP_1Pt);

    pDC->Select( pPen);

    pDC->MoveToEx( Column( IDColP_ItemNo)->Right( ), rectPos.lTop);
    pDC->LineTo( Column( IDColP_ItemNo)->Right( ), rectPos.lBottom);

    pDC->MoveToEx( Column( IDColP_Quantity)->Right( ), rectPos.lTop);
    pDC->LineTo( Column( IDColP_Quantity)->Right( ), rectPos.lBottom);

    pDC->MoveToEx( Column( IDColP_Code)->Right( ), rectPos.lTop);
    pDC->LineTo( Column( IDColP_Code)->Right( ), rectPos.lBottom);

    pDC->MoveToEx( Column( IDColP_Desc)->Right( ), rectPos.lTop);
    pDC->LineTo( Column( IDColP_Desc)->Right( ), rectPos.lBottom);

    pDC->MoveToEx( Column( IDColP_UnitPrice)->Right( ), rectPos.lTop);
    pDC->LineTo( Column( IDColP_UnitPrice)->Right( ), rectPos.lBottom);

    if (nItem >= 0)
      {
      pDC->MoveToEx( Column( IDColP_ItemNo)->Left( ORINSIDE), rectPos.lTop);
      pDC->LineTo( Column( IDColP_RightHalf)->Right( ), rectPos.lTop);
      }
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_Items::ORLayer_Items( ORReport_PackList *pRepPackList_,
                              PackingList *pPackList_)
              :ORLayer( pRepPackList_, 0, 6, 6)
  {
  pPackList = pPackList_;
  }

long ORLayer_Items::InstantiateChildren( )
  {
  long lErr = 0;

  for (int i=-1; i<pPackList->nItems+3 && !lErr; i++)
    {
    ORLayer_Item *pL = new ORLayer_Item( this, pPackList, i);
    if (pL)
      lErr = pL->Instantiate( );
    }

  return lErr;
  }

long ORLayer_Items::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    ORRect  r;
    memcpy( &r, &rectPos, sizeof( r));
    r.lTop    -= MarginTop( );
    r.lBottom += MarginBottom( );

    ORPen *pPen = Pen( IDPenP_2PtsInside);

    pDC->Select( pPen);
    pDC->SelectStockBrush( HOLLOW_BRUSH);
    pDC->RoundRect( r.lLeft, r.lTop, r.lRight, r.lBottom, 60, 60);

    long lW = pPen->Width( );
    pDC->RoundRect( r.lLeft + lW * 2,  r.lTop + lW * 2,
                    r.lRight - lW * 2, r.lBottom - lW * 2);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_Total::ORLayer_Total( ORReport_PackList *pRepPackList_,
                              PackingList *pPackList_)
              :ORLayer( pRepPackList_, 0, 6, 6)
  {
  pPackList = pPackList_;
  }

long ORLayer_Total::InstantiateChildren( )
  {
  long lErr = 0;

  char  s[21];

  ORLayer *pL = new ORLayer( this);
    new ORText( pL, 0, "Merchandise Total", IDFntP_Arial12, ORJUST_LEFT, IDColP_LeftHalf);
    sprintf( s, "$%2.2f",  pPackList->MerchTotal( ));
    new ORText( pL, 0, s, IDFntP_Arial12, 0, IDColP_UnitPrice);

  pL = new ORLayer( this);
    new ORText( pL, 0, "Shipping & Handling", IDFntP_Arial12, ORJUST_LEFT, IDColP_LeftHalf);
    sprintf( s, "$%2.2f",  pPackList->dSandH);
    new ORText( pL, 0, s, IDFntP_Arial12, 0, IDColP_UnitPrice);

  pL = new ORLayer( this);
    new ORText( pL, 0, "Sales Tax", IDFntP_Arial12, ORJUST_LEFT, IDColP_LeftHalf);
    sprintf( s, "$%2.2f",  pPackList->dSalesTax);
    new ORText( pL, 0, s, IDFntP_Arial12, 0, IDColP_UnitPrice);

  pL = new ORLayer( this);
    new ORText( pL, 0, "ORDER TOTAL", IDFntP_Arial12, ORJUST_LEFT, IDColP_LeftHalf);
    sprintf( s, "$%2.2f",  pPackList->OrderTotal( ));
    new ORText( pL, 0, s, IDFntP_Arial12, 0, IDColP_UnitPrice);

  return lErr;
  }

long ORLayer_Total::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    ORRect  r;
    memcpy( &r, &rectPos, sizeof( r));
    r.lTop    -= MarginTop( );
    r.lBottom += MarginBottom( );

    ORPen *pPen = Pen( IDPenP_2PtsInside);

    pDC->Select( pPen);
    pDC->SelectStockBrush( HOLLOW_BRUSH);
    pDC->RoundRect( r.lLeft, r.lTop, r.lRight, r.lBottom, 60, 60);

    long lW = pPen->Width( );
    pDC->RoundRect( r.lLeft + lW * 2,  r.lTop + lW * 2,
                    r.lRight - lW * 2, r.lBottom - lW * 2);
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_CompanyInfo::ORLayer_CompanyInfo( ORReport_PackList *pRepPackList_)
                    :ORLayer( pRepPackList_, 0, 48)
  {
  }

long ORLayer_CompanyInfo::InstantiateChildren( )
  {
  ORLayer *pL = new ORLayer( this);
    new ORText( pL, 0, "Rollins Software Inc", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "3091 Rhodenhaven Dr NW", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "Atlanta, GA  30327-1225", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "fax 404.352.9345", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "kentr@rollinssoft.com", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "72040.1607@compuserve.com", IDFntP_Arial16, ORPOS_CENTER);
  pL = new ORLayer( this);
    new ORText( pL, 0, "www.rollinssoft.com", IDFntP_Arial16, ORPOS_CENTER);

  return 0;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORReport_PackList::ORReport_PackList( char *psTitle, ABORTPROC lpfnAbortProc_,
                                      HWND hDlgPrint_)
                  :ORReport( psTitle, lpfnAbortProc_)
  {
  hDlgPrint = hDlgPrint_;
  }

long ORReport_PackList::Initialize( )
  {
  SendMessage( hDlgPrint, WMU_TITLE, 0, (LPARAM)sTitle);

  long lErr = ORReport::Initialize( );

  if (!lErr)
    pDC->SetBkMode( TRANSPARENT);

  return lErr;
  }

long ORReport_PackList::InstantiateColumns( )
  {
  //---- column group 1
  new ORColumn( this, IDColP_LeftHalf,  280, 8, 0, 0,               ORPOS_LEFT);
  new ORColumn( this, IDColP_RightHalf, 282, 0, 0, IDColP_LeftHalf, ORPOS_RIGHT);

  //---- column group 2, independent of column group 1
  new ORColumn( this, IDColP_ItemNo,     42, 6, 0, 0,                ORPOS_LEFT);
  new ORColumn( this, IDColP_Quantity,   42, 0, 0, IDColP_ItemNo,    ORPOS_RIGHT);
  new ORColumn( this, IDColP_Code,       60, 0, 0, IDColP_Quantity,  ORPOS_RIGHT);
  new ORColumn( this, IDColP_Desc,      168, 3, 3, IDColP_Code,      ORPOS_RIGHT);
  new ORColumn( this, IDColP_UnitPrice,  51, 0, 3, IDColP_Desc,      ORPOS_RIGHT);
  new ORColumn( this, IDColP_Notes,     186, 3, 9, IDColP_UnitPrice, ORPOS_RIGHT);

  return 0;
  }

/*-----------------------------------------------------------------------------
  these long, detailed calls to create ORFonts are not always required.  they
    are used here because the ORDemo.Exe program can be run on a variety of
    different machines with different fonts available.  by providing the
    Windows font mapper with the maximum amount of information, it will be
    able to provide a similar font if the desired font is not available.
    if you know ahead of time what fonts will be available when your reports
    are printed, you can simply create your fonts in the
    ORReport::InstantiateFonts member function as follows:
      new ORFont( this, IDFntA_Arial10,  "Arial", 10.0);
      new ORFont( this, IDFntA_Arial10B, "Arial", 10.0, 0, 0, 0, FW_BLACK);
-----------------------------------------------------------------------------*/
long ORReport_PackList::InstantiateFonts( )
  {
  new ORFont( this, IDFntP_Arial10, "Arial", 10.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntP_Arial12, "Arial", 12.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntP_Arial16, "Arial", 16.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntP_Arial24, "Arial", 24.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));

  return 0;
  }

long ORReport_PackList::InstantiatePens( )
  {
  new ORPen( this, IDPenP_2PtsInside, PS_INSIDEFRAME, 0, RGB( 0, 0, 0), 2);
  new ORPen( this, IDPenP_1Pt,        PS_SOLID,       0, RGB( 0, 0, 0), 1);

  return 0;
  }

long ORReport_PackList::InstantiateChildren( )
  {
  long lErr = 0;

  if (!lErr)
    {
    PackingList  oPackList;

    ORLayer *pL = new ORLayer_PTitle( this);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_Spacer( this, 1, &oPackList, 16, 16);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_BillShip( this, &oPackList);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_Spacer( this, 2, NULL, 6, 6);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_Items( this, &oPackList);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_Spacer( this, 3, NULL, 12, 0);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_Total( this, &oPackList);
    if (pL)
      lErr = pL->Instantiate( );

    pL = new ORLayer_CompanyInfo( this);
    if (pL)
      lErr = pL->Instantiate( );
    }

  return lErr;
  }

long ORReport_PackList::PrintPage( )
  {
  char  s[51];
  sprintf( s, "Printing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  long lErr = ORReport::PrintPage( );

  sprintf( s, "Initializing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  return lErr;
  }
