/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#include "orcc.h"
#include <windows.h>
#pragma  hdrstop

#include <string.h>
#include "ordemo.h"
#include "orfont.h"
#include "orprntdc.h"
#include "orsource.h"
#include "ortext.h"


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_SrcHeader::ORLayer_SrcHeader( ORReport_Source *pRepSrc_, long lAttr_)
                  :ORLayer( pRepSrc_, lAttr_)
  {
  pRepSrc = pRepSrc_;
  }

long ORLayer_SrcHeader::InstantiateChildren( )
  {
  long lErr = 0;

  ORLayer *pL = new ORLayer( this);
  new ORText( pL, 0, pRepSrc->File( ), IDFntSrc_Arial24B, ORPOS_LEFT);

  char  s[21];
  sprintf( s, "Page %d", pRepSrc->Page( ));
  new ORText( pL, 0, s, IDFntSrc_Arial12B, ORPOS_RIGHT | ORPOS_TOP);

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_10Lines::ORLayer_10Lines( ORLayer *pL, ORReport_Source *pRepSrc_)
                :ORLayer( pL)
  {
  pRepSrc = pRepSrc_;

  CheckPageFull( TRUE);
  }

long ORLayer_10Lines::InstantiateChildren( )
  {
  int   n = 1;
  char  s[201];
  while (pRepSrc->GetLine( s, sizeof( s)))
    {
    ORLayer *pL = new ORLayer( this);
    new ORText( pL, 0, s, IDFntSrc_CourNew9, ORPOS_LEFT);

    sprintf( s, "%s.%d", psId, n);
    pL->StringId( s);

    if (n == 10)
      break;

    n++;
    }

  return 0;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_Source::ORLayer_Source( ORReport_Source *pRepSrc_)
               :ORLayer( pRepSrc_)
  {
  pRepSrc = pRepSrc_;

  CheckPageFull( TRUE);
  }

long ORLayer_Source::InstantiateChildren( )
  {
  long lErr = 0;

  int n = 1;

  while ((!lErr) && (pRepSrc->EndOfFile( ) == FALSE))
    {
    ORLayer *pL = new ORLayer_10Lines( this, pRepSrc);

    char s[31];
    sprintf( s, "Layer %d", n++);
    pL->StringId( s);

    if (pL == NULL)
      return APPERR_MEMORY;

    lErr = pL->Instantiate( );
    }

  return lErr;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORReport_Source::ORReport_Source( char *psTitle, ABORTPROC lpfnAbortProc_,
                                  HWND hDlgPrint_, char *psFile)
                :ORReport( psTitle, lpfnAbortProc_)
  {
  hDlgPrint = hDlgPrint_;

  strcpy( sFile, psFile);
  }

long ORReport_Source::Initialize( )
  {
  SendMessage( hDlgPrint, WMU_TITLE, 0, (LPARAM)sTitle);

  long lErr = ORReport::Initialize( );

  return lErr;
  }

long ORReport_Source::Instantiate( ORPrintDC *pPrintDC)
  {
  long lErr = 0;

  //---- verify data file exists
  FILE *pF = fopen( sFile, "rt");
  if (pF == NULL)
    {
    char  s[101];
    sprintf( s, "Required data file '%s' does not exist.", sFile);
    MessageBox( hDlgPrint, s, NULL, MB_ICONSTOP);
    lErr = APPERR_FILE;
    }

  if (pF)
    fclose( pF);

  if (!lErr)
    lErr = ORReport::Instantiate( pPrintDC);

  return lErr;
  }

/*-----------------------------------------------------------------------------
  these long, detailed calls to create ORFonts are not always required.  they
    are used here because the ORDemo.Exe program can be run on a variety of
    different machines with different fonts available.  by providing the
    Windows font mapper with the maximum amount of information, it will be
    able to provide a similar font if the desired font is not available.
    if you know ahead of time what fonts will be available when your reports
    are printed, you can simply create your fonts in the
    ORReport::InstantiateFonts member function as follows:
      new ORFont( this, IDFntA_Arial10,  "Arial", 10.0);
      new ORFont( this, IDFntA_Arial10B, "Arial", 10.0, 0, 0, 0, FW_BLACK);
-----------------------------------------------------------------------------*/
long ORReport_Source::InstantiateFonts( )
  {
  new ORFont( this, IDFntSrc_Arial12B, "Arial", 12.0, 0.0, 0, 0, FW_BLACK,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntSrc_Arial24B, "Arial", 24.0, 0.0, 0, 0, FW_BLACK,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntSrc_CourNew9, "Courier New", 9.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (FIXED_PITCH | FF_SWISS));

  return 0;
  }

long ORReport_Source::InstantiateChildren( )
  {
  long lErr = 0;

  pFILE = fopen( sFile, "rt");
  if (pFILE == NULL)
    lErr = APPERR_FILE;

  if (!lErr)
    {
    ORLayer *pL = new ORLayer_Source( this);
    pL->StringId( "Body");
    if (pL)
      lErr = pL->Instantiate( );
    }

  fclose( pFILE);

  return lErr;
  }

char *ORReport_Source::GetLine( char *ps, int nChars)
  {
  return fgets( ps, nChars, pFILE);
  }

int ORReport_Source::EndOfFile( )
  {
  return feof( pFILE);
  }

long ORReport_Source::InstantiatePageLayers( )
  {
  long lErr = 0;

  //---- add header page layer
  if (!lErr)
    {
    ORLayer *pL = new ORLayer_SrcHeader( this, ORPOS_TOP);
    if (pL)
      lErr = pL->Instantiate( );
    }

  //---- add footer page layer
  if (!lErr)
    {
    }

  return lErr;
  }

long ORReport_Source::PrintPage( )
  {
  char  s[51];
  sprintf( s, "Printing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  long lErr = ORReport::PrintPage( );

  sprintf( s, "Initializing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  return lErr;
  }
