/*
[]===========================================================================[]
||                                                                           ||
||                            Object Reports 1.1                             ||
||                (C) Copyright RollinSoftware Inc 1995,1996                 ||
||                                                                           ||
[]===========================================================================[]
*/

#include "orcc.h"
#include <windows.h>
#pragma  hdrstop

#include <stdio.h>
#include "ordemo.h"
#include "orfont.h"
#include "orprntdc.h"
#include "ortext.h"
#include "ortulayr.h"


//-----------------------------------------------------------------------------
char *psPage = "Page Layers appear at the top (Headers) or bottom (Footers)"
               " of each Page.\n"
               "Page Layers are created in the InstantiatePageLayers()"
               " member function of your derived ORReport class.\n"
               "Create a Page Layer with an Attribute of ORPOS_TOP to create"
               " a header or ORPOS_BOTTOM to create a footer.\n"
               "You may create as many Page Layers for a Report as you"
               " wish.";
char *psBody = "Body Layers make up the body of the Report.  Body Layers are"
               " the same as Page Layers except they get created by"
               " ORReport::InstantiateChildren() and they have no"
               " Attributes.  An Object Reports Layer can contain objects"
               " derived from ORElement (such as ORText) or it can contain"
               " more ORLayers but not both types of objects.  Layers provide"
               " a mechanism for automatic vertical positioning of Report"
               " elements.  Layers also provide for pagination.  If a Layer is"
               " so big that it won't fit in the remaining space on a Page,"
               " Object Reports will carry that Layer to the next Page.  You"
               " can also define a Layer as Unbreakable, which means that if"
               " some of it's Elements won't fit on a Page, the whole Layer"
               " will be carried to the next Page.  Page Layers are"
               " optional, but you must define atleast one Body Layer if you"
               " want to print anything.";
char *psHeight = "A Layer's Height is determined by the Height of its Child"
                 " Layers just as the Height of the Body Layer on this page"
                 " is determined by the Height of all these Layers or if a"
                 " Layer contains Elements, its Height is determined by";
char *psWidth = "A Layer's Width is the Width of the printable area of the"
                " Device Context.";
char *psMargin = "Layers can also have Top and Bottom Margins.";
char *psDerive = "You can derive a new class from the ORLayer base class in"
                 " order to define new behavior for your Report.  For"
                 " example, every Layer on this page is an instance of"
                 " ORLayer_RoundRect (which was created especially for this"
                 " report) which automatically draws a rounded rectangle"
                 " around itself.  Normally, an application would use derived"
                 " functionality to create Layers that do specific things on"
                 " your Reports.  For example, the Sales by Month Demo"
                 " Report creates a Layer to represent a Year, a Layer to"
                 " represent a Month, and a Layer to represent a Product"
                 " sold.  The Year Layer is responsible for creating its"
                 " child Month Layers.  After each Month Layer finishes"
                 " instantiating itself, the Child Month Layer calls the"
                 " Parent Year Layer to tell the Year Layer the total sales"
                 " for the Month.  The Year Layer accumulates these Monthly"
                 " totals and reports the grand total at the end.  Notice how"
                 " the Month Layer does not look at the details of the Product"
                 " sold, it only determines that there is a product to be"
                 " reported, creates a Product Layer and passes the Product"
                 " object to the new Product Layer.  The Sales by Month Demo"
                 " Report is a fairly traditional type of Report where there"
                 " are a few types of layers that are basically repeated many"
                 " times.  While all the Layers of the Sales Report look alike,"
                 " you could easily code special behavior into the Layers to"
                 " perform specific tasks based on the data passed to the"
                 " Layer (ie, providing more or less detail based on the"
                 " data available).  In contrast to the Sales Report where"
                 " the Layers are repeated, every Layer of the Packing List"
                 " Demo Report is specialized and is used only once.";

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_RoundRect::ORLayer_RoundRect( ORReport *pR, long lAttr_,
                                      short shMargTop)
                  :ORLayer( pR, lAttr_, shMargTop)
  {
  }

ORLayer_RoundRect::ORLayer_RoundRect( ORLayer *pL, short shMargTop,
                                      short shMargBottom)
                  :ORLayer( pL, shMargTop, shMargBottom)
  {
  }

long ORLayer_RoundRect::PostPrint( ORDC *pDC)
  {
  long lErr = ORLayer::PostPrint( pDC);

  if (!lErr)
    {
    pDC->SelectStockPen( BLACK_PEN);
    pDC->SelectStockBrush( HOLLOW_BRUSH);
    pDC->RoundRect( rectPos.lLeft, rectPos.lTop,
                    rectPos.lRight, rectPos.lBottom,
                    30, 30);
    }

  return lErr;
  }

long ORLayer_RoundRect::SizeAndPosition_Elems_CalcHt( )
  {
  for (ORElement *pE=(ORElement *)pChildren; pE!=NULL; pE=pE->Next( ))
    {
    long lHeightRel = pE->HeightRel( );
    if (lHeightRel > Height( ORINSIDE))
      rectPos.lBottom = rectPos.lTop + lHeightRel;
    }

  return 0;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORLayer_LTBody::ORLayer_LTBody( ORReport *pR)
               :ORLayer_RoundRect( pR, 0, 36)
  {
  }

long ORLayer_LTBody::InstantiateChildren( )
  {
  ORLayer *pL = new ORLayer_RoundRect( this);
    new ORText( pL, 0, "This is a Body Layer", IDFntTL_Arial18, ORPOS_CENTER);
  pL = new ORLayer_RoundRect( this);
    ORText *pT = new ORText( pL, 0, psBody, IDFntTL_Arial10, ORPOS_LEFT);
    pT->SetMaxWidth( pL);
  pL = new ORLayer_RoundRect( this);
    pT = new ORText( pL, 0, psHeight, IDFntTL_Arial10, ORPOS_LEFT);
    pT->SetMaxWidth( pL);
  pL = new ORLayer_RoundRect( this);
    new ORText( pL, 1, "the Height ", IDFntTL_Arial10, ORPOS_LEFT,  0);
    new ORText( pL, 2, "of the ",     IDFntTL_Arial24, ORPOS_RIGHT | ORPOS_MIDDLE | ORJUST_BOTTOM, 1);
    new ORText( pL, 3, "tallest ",    IDFntTL_Arial32, ORPOS_RIGHT | ORPOS_MIDDLE | ORJUST_BOTTOM, 2);
    new ORText( pL, 4, "Child ",      IDFntTL_Arial12, ORPOS_RIGHT | ORPOS_MIDDLE | ORJUST_BOTTOM, 3);
    new ORText( pL, 0, "Element.",    IDFntTL_Arial18, ORPOS_RIGHT | ORPOS_MIDDLE | ORJUST_BOTTOM, 4);
  pL = new ORLayer_RoundRect( this);
    new ORText( pL, 0, psWidth, IDFntTL_Arial10, ORPOS_LEFT);
  pL = new ORLayer_RoundRect( this, 24, 24);
    new ORText( pL, 0, psMargin, IDFntTL_Arial10, ORPOS_LEFT);
  pL = new ORLayer_RoundRect( this);
    pT = new ORText( pL, 0, psDerive, IDFntTL_Arial10, ORPOS_LEFT);
    pT->SetMaxWidth( pL);

  return 0;
  }


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ORReport_LayerTutor::ORReport_LayerTutor( char *psTitle,
                                          ABORTPROC lpfnAbortProc_,
                                          HWND hDlgPrint_)
                    :ORReport( psTitle, lpfnAbortProc_)
  {
  hDlgPrint = hDlgPrint_;
  }

long ORReport_LayerTutor::Initialize( )
  {
  SendMessage( hDlgPrint, WMU_TITLE, 0, (LPARAM)sTitle);

  long lErr = ORReport::Initialize( );

  return lErr;
  }

/*-----------------------------------------------------------------------------
  these long, detailed calls to create ORFonts are not always required.  they
    are used here because the ORDemo.Exe program can be run on a variety of
    different machines with different fonts available.  by providing the
    Windows font mapper with the maximum amount of information, it will be
    able to provide a similar font if the desired font is not available.
    if you know ahead of time what fonts will be available when your reports
    are printed, you can simply create your fonts in the
    ORReport::InstantiateFonts member function as follows:
      new ORFont( this, IDFntA_Arial10,  "Arial", 10.0);
      new ORFont( this, IDFntA_Arial10B, "Arial", 10.0, 0, 0, 0, FW_BLACK);
-----------------------------------------------------------------------------*/
long ORReport_LayerTutor::InstantiateFonts( )
  {
  new ORFont( this, IDFntTL_Arial10, "Arial", 10.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntTL_Arial12, "Arial", 12.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntTL_Arial18, "Arial", 18.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntTL_Arial24, "Arial", 24.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));
  new ORFont( this, IDFntTL_Arial32, "Arial", 32.0, 0.0, 0, 0, FW_NORMAL,
              0, 0, 0, 0, OUT_DEFAULT_PRECIS,
              CLIP_DEFAULT_PRECIS, PROOF_QUALITY, (VARIABLE_PITCH | FF_SWISS));

  return 0;
  }

long ORReport_LayerTutor::InstantiateChildren( )
  {
  long lErr = 0;

  if (!lErr)
    {
    ORLayer *pL = new ORLayer_LTBody( this);
    if (pL)
      lErr = pL->Instantiate( );
    }

  return lErr;
  }

long ORReport_LayerTutor::InstantiatePageLayers( )
  {
  long lErr = 0;

  if (!lErr)
    {
    ORLayer *pL = new ORLayer_RoundRect( this, ORPOS_TOP);
    if (pL)
      new ORText( pL, 0, "This is a Page Layer", IDFntTL_Arial18, ORPOS_CENTER);

    pL = new ORLayer_RoundRect( this, ORPOS_TOP);
    if (pL)
      new ORText( pL, 0, psPage, IDFntTL_Arial10, ORPOS_CENTER);

    pL = new ORLayer_RoundRect( this, ORPOS_BOTTOM);
    if (pL)
      new ORText( pL, 0, "This is a Page Layer", IDFntTL_Arial18, ORPOS_CENTER);
    }

  return lErr;
  }

long ORReport_LayerTutor::PrintPage( )
  {
  char  s[51];
  sprintf( s, "Printing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  long lErr = ORReport::PrintPage( );

  sprintf( s, "Initializing page %d", shPage);
  SendMessage( hDlgPrint, WMU_PRINTINGPAGE, 0, (LPARAM)s);

  return lErr;
  }
